'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tinycolor = require('@ctrl/tinycolor');
var fileType = _interopDefault(require('file-type'));
var Jimp = _interopDefault(require('jimp'));
var gifWrap = _interopDefault(require('gifwrap'));
var fs = _interopDefault(require('fs'));

function unhexlify(str) {
    var result = '';
    if (str.length % 2 !== 0) {
        throw new Error('The string length is not a multiple of 2');
    }
    for (var i = 0, l = str.length; i < l; i += 2) {
        var toHex = parseInt(str.substr(i, 2), 16);
        if (isNaN(toHex)) {
            throw new Error('str contains non hex character');
        }
        result += String.fromCharCode(toHex);
    }
    return result;
}
function int2hexlittle(value) {
    if (value > 65535 || value < 0) {
        throw new TypeError('int2hexlittle only supports value between 0 and 65535');
    }
    var byte1 = (value & 0xFF).toString(16).padStart(2, "0");
    var byte2 = ((value >> 8) & 0xFF).toString(16).padStart(2, "0");
    return "" + byte1 + byte2;
}

var DivoomTimeBoxEvoProtocol = /** @class */ (function () {
    function DivoomTimeBoxEvoProtocol() {
        this.PREFIX = "01";
        this.SUFFIX = "02";
        this._fullMessage = [];
        this._gifFrame = [];
    }
    DivoomTimeBoxEvoProtocol.prototype._setMessage = function (msg) {
        if (msg === undefined) {
            return;
        }
        var localmsg;
        if (Buffer.isBuffer(msg)) {
            localmsg = msg.toString('ascii');
        }
        else {
            localmsg = msg.toLowerCase();
        }
        this._length = int2hexlittle((msg.length + 4) / 2);
        this._setCRC(this._length + localmsg);
        this._message = localmsg;
        var fullMessage = [];
        var message = this.PREFIX + this._length + this._message + this._crc + this.SUFFIX;
        if (message) {
            message.match(/.{1,1332}/g).forEach(function (slice) {
                fullMessage.push(slice);
            });
        }
        this._fullMessage = fullMessage;
    };
    DivoomTimeBoxEvoProtocol.prototype._queueMessage = function (msg) {
        var _this = this;
        if (msg === undefined) {
            return;
        }
        var localmsg;
        if (Buffer.isBuffer(msg)) {
            localmsg = msg.toString('ascii');
        }
        else {
            localmsg = msg.toLowerCase();
        }
        this._length = int2hexlittle((msg.length + 4) / 2);
        this._setCRC(this._length + localmsg);
        this._message = localmsg;
        var message = this.PREFIX + this._length + this._message + this._crc + this.SUFFIX;
        if (message) {
            message.match(/.{1,1332}/g).forEach(function (slice) {
                _this._fullMessage.push(slice);
            });
        }
    };
    DivoomTimeBoxEvoProtocol.prototype._setCRC = function (msg) {
        if (msg !== undefined) {
            var sum = 0;
            for (var i = 0, l = msg.length; i < l; i += 2) {
                sum += parseInt(msg.substr(i, 2), 16);
            }
            this._crc = int2hexlittle(sum % 65536);
        }
    };
    DivoomTimeBoxEvoProtocol.prototype._string2BinBuffer = function (msg) {
        if (msg === undefined) {
            return undefined;
        }
        var bufferArray = [];
        msg.forEach(function (slice) {
            bufferArray.push(new Buffer(unhexlify(slice), 'binary'));
        });
        return bufferArray;
    };
    DivoomTimeBoxEvoProtocol.prototype._boolean2HexString = function (bool) {
        return bool ? "01" : "00";
    };
    DivoomTimeBoxEvoProtocol.prototype._number2HexString = function (int) {
        return int.toString(16).padStart(2, "0");
    };
    DivoomTimeBoxEvoProtocol.prototype._color2HexString = function (color) {
        return color.toHexString().substring(1);
    };
    DivoomTimeBoxEvoProtocol.prototype._brightness2HexString = function (brightness) {
        if (brightness > 100) {
            brightness = 100;
        }
        else if (brightness < 0) {
            brightness = 0;
        }
        return this._number2HexString(Math.trunc(brightness));
    };
    /**
     * Generates the appropriate message to display the Time Channel on the Divoom Timebox Evo
     * @param type type of time to display:
     * @param color the color to display, can be of any type supported by [TinyColor]
     * @param showTime show the time?
     * @param showWeather show the weather?
     * @param showTemp show the temperature?
     * @param showCalendar show the calendar?
     */
    DivoomTimeBoxEvoProtocol.prototype.displayTimePackage = function (type, color, showTime, showWeather, showTemp, showCalendar) {
        if (type === void 0) { type = 0; }
        if (color === void 0) { color = "#FFFFFF"; }
        if (showTime === void 0) { showTime = true; }
        if (showWeather === void 0) { showWeather = false; }
        if (showTemp === void 0) { showTemp = false; }
        if (showCalendar === void 0) { showCalendar = false; }
        var PACKAGE_PREFIX = "450001";
        var localColor = new tinycolor.TinyColor(color);
        if (!localColor.isValid) {
            throw new Error("Provided color " + color + " is not valid");
        }
        this._setMessage(PACKAGE_PREFIX
            + this._number2HexString(type)
            + this._boolean2HexString(showTime)
            + this._boolean2HexString(showWeather)
            + this._boolean2HexString(showTemp)
            + this._boolean2HexString(showCalendar)
            + this._color2HexString(localColor));
    };
    /**
     * Generates the appropriate message to display the Lightning Channel on the Divoom Timebox Evo
     * @param type type of lightning you want to display
     * @param color the color to display, can be of any type supported by [TinyColor]
     * @param brightness brightness `0` - `100` range
     * @param power if `false`, the display is turned off
     */
    DivoomTimeBoxEvoProtocol.prototype.displayLightningPackage = function (type, color, brightness, power) {
        if (type === void 0) { type = 0; }
        if (color === void 0) { color = "#FFFFFF"; }
        if (brightness === void 0) { brightness = 100; }
        if (power === void 0) { power = true; }
        var PACKAGE_PREFIX = "4501";
        var PACKAGE_SUFFIX = "000000";
        var localColor = new tinycolor.TinyColor(color);
        if (!localColor.isValid) {
            throw "Provided color " + color + " is not valid";
        }
        this._setMessage(PACKAGE_PREFIX
            + this._color2HexString(localColor)
            + this._brightness2HexString(brightness)
            + this._number2HexString(type)
            + this._boolean2HexString(power)
            + PACKAGE_SUFFIX);
    };
    /**
     * Generates the appropriate message to display the Cloud Channel
     */
    DivoomTimeBoxEvoProtocol.prototype.displayCloudPackage = function () {
        this._setMessage("4502");
    };
    /**
     * Generates the appropriate message to display the VJ Effects Channel
     * @param type type of Effect to display
     */
    DivoomTimeBoxEvoProtocol.prototype.displayVJEffectsPackage = function (type) {
        if (type === void 0) { type = 0; }
        var PACKAGE_PREFIX = "4503";
        this._setMessage(PACKAGE_PREFIX
            + this._number2HexString(type));
    };
    /**
     * Generates the appropriate message to display the scoreboard
     * @param red the score for the red player (0 - 999)
     * @param blue the score for the blue player (0 - 999)
     */
    DivoomTimeBoxEvoProtocol.prototype.displayScoreBoardPackage = function (red, blue) {
        if (red === void 0) { red = 0; }
        if (blue === void 0) { blue = 0; }
        var PACKAGE_PREFIX = "450600";
        red = Math.min(999, Math.max(0, red));
        blue = Math.min(999, Math.max(0, blue));
        this._setMessage(PACKAGE_PREFIX
            + int2hexlittle(red)
            + int2hexlittle(blue));
    };
    /**
     * Generates the appropriate message to set the temperature and the weather
     * @param temp temperature to set (`-127 <= temp <= 128`)
     * @param weatherType type of weather to set
     */
    DivoomTimeBoxEvoProtocol.prototype.setTempAndWeatherPackage = function (temp, weatherType) {
        var PACKAGE_PREFIX = "5F";
        if (temp > 128 || temp < -127) {
            throw new Error('temp should be >= -127 and <= 128');
        }
        var encodedTemp = "";
        if (temp >= 0) {
            encodedTemp = this._number2HexString(temp);
        }
        else {
            var value = 256 + temp;
            encodedTemp = this._number2HexString(value);
        }
        this._setMessage(PACKAGE_PREFIX
            + encodedTemp
            + this._number2HexString(weatherType));
    };
    /**
     * Generates the appropiate message to set the brightness on the Timebox
     * @param brightness brightness (`0 - 100`) if `in_min` or `in_max` are undefined, else `in_min <= brightness <= in_max`
     * @param in_min
     * @param in_max
     */
    DivoomTimeBoxEvoProtocol.prototype.setBrightness = function (brightness, in_min, in_max) {
        var PACKAGE_PREFIX = "74";
        function map(x, in_min, in_max, out_min, out_max) {
            if (x < in_min || x > in_max) {
                throw new Error('map() in_min is < value or in_max > value');
            }
            return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;
        }
        var briInRange = brightness;
        if (in_min !== undefined && in_max !== undefined) {
            briInRange = Math.ceil(map(brightness, in_min, in_max, 0, 100));
        }
        if ((brightness > 100 || brightness < 0) && (in_min === undefined || in_max === undefined)) {
            throw new Error('Brightness should be between 0 and 100 or in_min and in_max should be defined');
        }
        this._setMessage(PACKAGE_PREFIX
            + this._number2HexString(briInRange));
    };
    /**
     * Generates the full message, properly split which will be ready to be send to Divoom
     * @param msg the string or Buffer that you want to encode before sending to Divoom
     */
    DivoomTimeBoxEvoProtocol.prototype.createRAWPackage = function (msg) {
        this._setMessage(msg);
    };
    /**
     * @returns The an array of strings of all the messages which should be sent
     */
    DivoomTimeBoxEvoProtocol.prototype.getDivoomMessageArray = function () {
        return this._fullMessage;
    };
    DivoomTimeBoxEvoProtocol.prototype.getDivoomMessageString = function () {
        if (this._fullMessage.length === 0)
            return undefined;
        var message = "";
        this._fullMessage.forEach(function (elt) {
            message += elt;
        });
        return message;
    };
    /**
     * @returns The an array of messages as a Binary Buffer ready to be sent
     */
    DivoomTimeBoxEvoProtocol.prototype.getDivoomBinaryBuffer = function () {
        return this._string2BinBuffer(this._fullMessage);
    };
    /**
     * Generates the appropriate message to display an animation or an image on the Timebox
     * @param input a path to an image or a Buffer representing an image file
     * @returns A promise which resolves when the processing is done
     */
    DivoomTimeBoxEvoProtocol.prototype.displayAnimation = function (input) {
        this._fullMessage = [];
        var buffer = fs.readFileSync(input);
        var ft = fileType(buffer);
        ft = fileType(buffer);
        if (ft) {
            switch (ft.mime) {
                case 'image/gif':
                    return this._displayAnimationFromGIF(buffer);
                case 'image/jpeg':
                case 'image/png':
                case 'image/bmp':
                    return this._displayImage(buffer);
                default:
                    throw new Error('file type not supported');
            }
        }
        else {
            throw new Error('file type unkown');
        }
    };
    /**
     * This function generates the message when the a static image is used
     * @param input a Buffer representing an image file
     * @returns A promise which resolves when the processing is done
     */
    DivoomTimeBoxEvoProtocol.prototype._displayImage = function (input) {
        var _this = this;
        var PACKAGE_PREFIX = '44000A0A04AA';
        return new Promise(function (resolve, reject) {
            var promise = Jimp.read(input);
            promise.then(function (image) {
                var resized = image.resize(16, 16, Jimp.RESIZE_NEAREST_NEIGHBOR);
                var colorsHash = [];
                var colorArray = [];
                var counter = 0;
                var pixelArray = [];
                resized.scan(0, 0, resized.bitmap.width, resized.bitmap.height, function (x, y, idx) {
                    var red = this.bitmap.data[idx + 0];
                    var green = this.bitmap.data[idx + 1];
                    var blue = this.bitmap.data[idx + 2];
                    // let alpha = this.bitmap.data[idx + 3];
                    var color = (red << 16) + (green << 8) + blue;
                    if (!colorsHash[color] && colorsHash[color] !== 0) {
                        colorsHash[color] = counter;
                        colorArray.push(color);
                        pixelArray[x + 16 * y] = counter;
                        counter++;
                    }
                    else {
                        pixelArray[x + 16 * y] = colorsHash[color];
                    }
                });
                var nbColors = (counter % 256).toString(16).padStart(2, "0");
                var colorString = '';
                colorArray.forEach(function (color) {
                    colorString += color.toString(16).padStart(6, '0');
                });
                var nbBitsForAPixel = Math.log(counter) / Math.log(2);
                var bits = Number.isInteger(nbBitsForAPixel)
                    ? nbBitsForAPixel
                    : (Math.trunc(nbBitsForAPixel) + 1);
                if (bits === 0)
                    bits = 1;
                var pixelString = '';
                pixelArray.forEach(function (pixel) {
                    pixelString += (pixel >>> 0).toString(2).padStart(8, '0').split("").reverse().join("").substring(0, bits);
                });
                var pixBinArray = pixelString.match(/.{1,8}/g);
                var pixelStringFinal = '';
                pixBinArray.forEach(function (pixel) {
                    pixelStringFinal += parseInt(pixel.split("").reverse().join(""), 2).toString(16).padStart(2, '0');
                });
                var length = int2hexlittle(('AA0000000000' + nbColors + colorString + pixelStringFinal).length / 2);
                _this._setMessage(PACKAGE_PREFIX
                    + length
                    + '000000'
                    + nbColors
                    + colorString
                    + pixelStringFinal);
                resolve(_this);
            })
                .catch(function (err) {
                reject(err);
            });
        });
    };
    /**
     * This function generates the message when the a static image is used
     * @param input Buffer representing an image file
     * @returns A promise which resolves when the processing is done
     */
    DivoomTimeBoxEvoProtocol.prototype._displayAnimationFromGIF = function (input) {
        var _this = this;
        var PACKAGE_PREFIX = '49';
        return new Promise(function (resolve, reject) {
            var gifCodec = new gifWrap.GifCodec();
            gifCodec.decodeGif(input).then(function (inputGif) {
                //node.send({width: inputGif.width});
                var frameNb = 0;
                var totalSize = 0;
                var encodedString = '';
                inputGif.frames.forEach(function (frame) {
                    var colorsArray = [];
                    var colorCounter = 0;
                    var frameColors = [];
                    var pixelArray = [];
                    var delay = frame.delayCentisecs * 10;
                    // to Fix ?
                    var resetPalette = true;
                    var image = gifWrap.GifUtil.copyAsJimp(Jimp, frame).resize(16, 16);
                    image.scan(0, 0, image.bitmap.width, image.bitmap.height, function (x, y, idx) {
                        // x, y is the position of this pixel on the image
                        // idx is the position start position of this rgba tuple in the bitmap Buffer
                        // this is the image
                        var red = this.bitmap.data[idx + 0];
                        var green = this.bitmap.data[idx + 1];
                        var blue = this.bitmap.data[idx + 2];
                        var color = (red << 16) + (green << 8) + blue;
                        if (!colorsArray.includes(color)) {
                            colorsArray.push(color);
                            frameColors.push(color);
                            pixelArray[x + 16 * y] = colorCounter;
                            colorCounter++;
                        }
                        else {
                            pixelArray[x + 16 * y] = colorsArray.findIndex(function (element) {
                                return element === color;
                            });
                        }
                    });
                    _this._gifFrame[frameNb] = {};
                    _this._gifFrame[frameNb].resetPalette = resetPalette;
                    _this._gifFrame[frameNb].pixelArray = pixelArray;
                    _this._gifFrame[frameNb].frameColors = frameColors;
                    _this._gifFrame[frameNb].nbColorsHex = (frameColors.length % 256).toString(16).padStart(2, "0");
                    var colorString = '';
                    frameColors.forEach(function (color) {
                        colorString += color.toString(16).padStart(6, '0');
                    });
                    _this._gifFrame[frameNb].colorString = colorString;
                    var whatever = Math.log(colorCounter) / Math.log(2);
                    var bits = Number.isInteger(whatever) ? whatever : (Math.trunc(whatever) + 1);
                    if (bits === 0)
                        bits = 1;
                    var pixelString = '';
                    pixelArray.forEach(function (pixel) {
                        pixelString += (pixel >>> 0).toString(2).padStart(8, '0').split("").reverse().join("").substring(0, bits);
                    });
                    var pixBinArray = pixelString.match(/.{1,8}/g);
                    var pixelStringFinal = '';
                    pixBinArray.forEach(function (pixel) {
                        pixelStringFinal += parseInt(pixel.split("").reverse().join(""), 2).toString(16).padStart(2, '0');
                    });
                    _this._gifFrame[frameNb].pixelString = pixelStringFinal;
                    _this._gifFrame[frameNb].frame = frameNb;
                    _this._gifFrame[frameNb].delay = delay;
                    _this._gifFrame[frameNb].delayHex = int2hexlittle(_this._gifFrame[frameNb].delay);
                    _this._gifFrame[frameNb].stringWithoutHeader =
                        _this._gifFrame[frameNb].delayHex +
                            ( "00" ) +
                            _this._gifFrame[frameNb].nbColorsHex +
                            _this._gifFrame[frameNb].colorString +
                            _this._gifFrame[frameNb].pixelString;
                    _this._gifFrame[frameNb].size = (_this._gifFrame[frameNb].stringWithoutHeader.length + 6) / 2;
                    totalSize += _this._gifFrame[frameNb].size;
                    _this._gifFrame[frameNb].sizeHex = int2hexlittle(_this._gifFrame[frameNb].size);
                    _this._gifFrame[frameNb].fullString =
                        'aa' +
                            _this._gifFrame[frameNb].sizeHex +
                            _this._gifFrame[frameNb].stringWithoutHeader;
                    encodedString += _this._gifFrame[frameNb].fullString;
                    frameNb++;
                });
                var messageCounter = 0;
                var totalSizeHex = int2hexlittle(totalSize);
                encodedString.match(/.{1,400}/g).forEach(function (message) {
                    _this._queueMessage(PACKAGE_PREFIX
                        + totalSizeHex
                        + messageCounter.toString(16).padStart(2, '0')
                        + message);
                    messageCounter++;
                });
                resolve(_this);
            })
                .catch(function (err) {
                reject(err);
            });
        });
    };
    return DivoomTimeBoxEvoProtocol;
}());

exports.DivoomTimeBoxEvoProtocol = DivoomTimeBoxEvoProtocol;
